@@ -26,15 +26,19 @@ test!(
 );
 
 test!(
-    assignment_sub,
+    assignment_arithmetic,
     r#"
-    {a: [0, 1], b: 2} | .["a"].[0] -= 1
+    .[] += 2, .[] -= 2, .[] *= 2, .[] /= 2, .[] %= 2
     "#,
     r#"
-    null
+    [1,2,3]
     "#,
     r#"
-    {"a": [-1, 1], "b": 2}
+    [3,4,5]
+    [-1,0,1]
+    [2,4,6]
+    [0.5,1,1.5]
+    [1,0,1]
     "#
 );
 
